<?php declare(strict_types=1);

namespace App\Http\Controllers\Admin\Projects;

use App\Dto\QuerySettingsDto;
use App\Http\Controllers\Admin\Controller;
use App\Http\Requests\Admin\Projects\DocumentVersions\IndexRequest;
use App\Http\Requests\Admin\Projects\DocumentVersions\StoreUpdateRequest;
use App\Services\Admin\Project\DocumentationVersionService;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\View\View;

final class DocumentationVersionController extends Controller
{
    public function __construct(
        private readonly DocumentationVersionService $documentationVersionService,
    ) { }

    public function index(int $projectId, IndexRequest $request): View
    {
        $user = $request->user();
        $data = $request->getDto();
        $querySettingsDto = new QuerySettingsDto(
            limit: 20,
            page: $data->getPage(),
            queryWith: []
        );

        $result = $this->documentationVersionService->index($projectId, $data->getDocumentationVersionBuildDto(), $querySettingsDto, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/documentation-versions/index', $result->getData());
    }

    public function show(int $projectId, int $id, Request $request): View
    {
        $user = $request->user();
        $result = $this->documentationVersionService->show($projectId, $id, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/documentation-versions/show', $result->getData());
    }

    public function create(int $projectId, Request $request): View
    {
        $user = $request->user();
        $result = $this->documentationVersionService->create($projectId, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/documentation-versions/create', $result->getData());
    }

    public function edit(int $projectId, int $id, Request $request): View
    {
        $user = $request->user();
        $result = $this->documentationVersionService->edit($projectId, $id, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/documentation-versions/edit', $result->getData());
    }

    public function store(int $projectId, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->documentationVersionService->store($projectId, $data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.documentation-versions.edit', [
            'project' => $projectId,
            'documentation_version' => $result->getModel()->id,
        ])->withSuccess($result->getMessage());
    }

    public function update(int $projectId, int $id, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->documentationVersionService->update($projectId, $id, $data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.documentation-versions.edit', [
            'project' => $projectId,
            'documentation_version' => $result->getModel()->id,
        ])->withSuccess($result->getMessage());
    }

    public function destroy(int $projectId, int $id, Request $request): RedirectResponse
    {
        $user = $request->user();
        $result = $this->documentationVersionService->destroy($projectId, $id, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.documentation-versions.index', ['project' => $projectId])->withSuccess($result->getMessage());
    }
}
