<?php declare(strict_types=1);

namespace App\Http\Controllers\Admin\Projects;

use App\Dto\QuerySettingsDto;
use App\Http\Controllers\Admin\Controller;
use App\Http\Requests\Admin\Projects\Documentations\IndexRequest;
use App\Http\Requests\Admin\Projects\Documentations\StoreUpdateRequest;
use App\Services\Admin\Project\DocumentationService;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\View\View;

final class DocumentationsController extends Controller
{
    public function __construct(
        private readonly DocumentationService $documentationService,
    ) { }

    public function index(int $projectId, int $versionId, IndexRequest $request): View
    {
        $user = $request->user();
        $data = $request->getDto();
        $querySettingsDto = new QuerySettingsDto(
            limit: 20,
            page: $data->getPage(),
            queryWith: []
        );

        $result = $this->documentationService->index($projectId, $versionId, $data->getDocumentation(), $querySettingsDto, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/documentations/index', $result->getData());
    }

    public function create(int $projectId, int $versionId, Request $request): View
    {
        $user = $request->user();
        $result = $this->documentationService->create($projectId, $versionId, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/documentations/create', $result->getData());
    }

    public function edit(int $projectId, int $versionId, int $id, Request $request): View
    {
        $user = $request->user();
        $result = $this->documentationService->edit($projectId, $versionId, $id, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/documentations/edit', $result->getData());
    }

    public function store(int $projectId, int $versionId, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->documentationService->store($projectId, $versionId, $data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.documentation-versions.documentations.edit', [
            'project' => $projectId,
            'version' => $versionId,
            'documentation' => $result->getModel()->id,
        ])->withSuccess($result->getMessage());
    }

    public function update(int $projectId, int $versionId, int $id, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->documentationService->update($projectId, $versionId, $id, $data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.documentation-versions.documentations.edit', [
            'project' => $projectId,
            'version' => $versionId,
            'documentation' => $result->getModel()->id,
        ])->withSuccess($result->getMessage());
    }

    public function destroy(int $projectId, int $versionId, int $id, Request $request): RedirectResponse
    {
        $user = $request->user();
        $result = $this->documentationService->destroy($projectId, $versionId, $id, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.documentation-versions.documentations.index', [
            'project' => $projectId,
            'version' => $versionId,
        ])->withSuccess($result->getMessage());
    }
}
