<?php declare(strict_types=1);

namespace App\Http\Controllers\Admin\Projects;

use App\Dto\QuerySettingsDto;
use App\Http\Controllers\Admin\Controller;
use App\Http\Requests\Admin\Projects\Links\IndexRequest;
use App\Http\Requests\Admin\Projects\Links\StoreUpdateRequest;
use App\Services\Admin\Project\LinkService;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\View\View;

final class LinksController extends Controller
{
    public function __construct(
        private readonly LinkService $linkService
    ) { }

    public function index(int $projectId, IndexRequest $request): View
    {
        $user = $request->user();
        $data = $request->getDto();
        $querySettingsDto = new QuerySettingsDto(
            limit: 20,
            page: $data->getPage(),
            queryWith: ['language']
        );

        $result = $this->linkService->index($projectId, $data->getLinkBuilderDto(), $querySettingsDto, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/links/index', $result->getData());
    }

    public function create(int $projectId, Request $request): View
    {
        $user = $request->user();
        $result = $this->linkService->create($projectId, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/links/create', $result->getData());
    }

    public function edit(int $projectId, int $id, Request $request): View
    {
        $user = $request->user();
        $result = $this->linkService->edit($projectId, $id, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/links/edit', $result->getData());
    }

    public function store(int $projectId, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->linkService->store($projectId, $data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.links.edit', [
            'project' => $projectId,
            'link'    => $result->getModel()->id,
        ])->withSuccess($result->getMessage());
    }

    public function update(int $projectId, int $id, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->linkService->update($projectId, $id, $data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.links.edit', [
            'project' => $projectId,
            'link'    => $result->getModel()->id,
        ])->withSuccess($result->getMessage());
    }

    public function destroy(int $projectId, int $id, Request $request): RedirectResponse
    {
        $user = $request->user();
        $result = $this->linkService->destroy($projectId, $id, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.links.index', ['project' => $projectId])->withSuccess($result->getMessage());
    }
}
