<?php declare(strict_types=1);

namespace App\Http\Controllers\Admin;

use App\Dto\QuerySettingsDto;
use App\Http\Requests\Admin\Projects\IndexRequest;
use App\Http\Requests\Admin\Projects\StoreUpdateRequest;
use App\Services\Admin\ProjectService;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\View\View;

final class ProjectsController extends Controller
{
    public function __construct(
        private readonly ProjectService $projectService
    ) { }

    public function index(IndexRequest $request): View
    {
        $user = $request->user();
        $data = $request->getDto();
        $querySettingsDto = new QuerySettingsDto(
            limit: 20,
            page: $data->getPage(),
            queryWith: []
        );

        $result = $this->projectService->index($data->getProjectBuilderDto(), $querySettingsDto, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/index', $result->getData());
    }

    public function show(int $id, Request $request): View
    {
        $user = $request->user();
        $result = $this->projectService->show($id, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/show', $result->getData());
    }

    public function create(Request $request): View
    {
        $user = $request->user();
        $result = $this->projectService->create($user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/create', $result->getData());
    }

    public function edit(int $id, Request $request): View
    {
        $user = $request->user();
        $result = $this->projectService->edit($id, $user);
        if ($result->isError()) {
            $this->errors($result);
        }

        return view('admin/projects/edit', $result->getData());
    }

    public function store(StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->projectService->store($data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.edit', $result->getModel())->withSuccess($result->getMessage());
    }

    public function update(int $id, StoreUpdateRequest $request): RedirectResponse
    {
        $data = $request->getDto();
        $user = $request->user();
        $result = $this->projectService->update($id, $data, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.edit', $result->getModel())->withSuccess($result->getMessage());
    }

    public function destroy(int $id, Request $request): RedirectResponse
    {
        $user = $request->user();
        $result = $this->projectService->destroy($id, $user);
        if ($result->isError()) {
            return redirect()->back()->withInput()->withErrors($result->getErrorsOrMessage());
        }

        return redirect()->route('admin.projects.index')->withSuccess($result->getMessage());
    }
}
