<?php declare(strict_types=1);

namespace App\Http\Requests\Admin\Projects\Documentations;

use App\Contracts\FormRequestDto;
use App\Dto\Service\Admin\Project\Documentation\StoreUpdate;
use App\Dto\Service\Admin\Project\DocumentationContent\Content;
use App\Dto\Service\Admin\Project\DocumentationContent\Contents;
use App\Dto\Service\Storage\Storages;
use App\Enums\StorageType;
use Illuminate\Foundation\Http\FormRequest;

final class StoreUpdateRequest extends FormRequest implements FormRequestDto
{
    public function attributes(): array
    {
        return [
            'content.*.content_images.*.file' => __('validation.attributes.content_images'),
        ];
    }

    /**
     * Get the validation rules that apply to the request.
     */
    public function rules(): array
    {
        return [
            'slug' => ['required', 'string', 'max:200', 'regex:/^[a-z0-9._-]+$/'],
            'is_public' => ['required', 'boolean'],
            'sort' => ['required', 'integer', 'min:-1000', 'max:1000'],
            'category_id' => ['nullable', 'integer', 'exists:documentation_categories,id'],
            'content.*.title' => ['required', 'string', 'max:255'],
            'content.*.content' => ['nullable', 'string'],
            'content.*.content_images.*.file' => ['numeric', 'min:1'],
            'translate-automatically.*' => ['nullable', 'boolean'],
        ];
    }

    public function getDto(): StoreUpdate
    {
        $categoryId = $this->input('category_id', null);
        if (!\is_null($categoryId)) {
            $categoryId = (int) $categoryId;
        }

        return new StoreUpdate(
            slug: $this->input('slug'),
            isPublic: (bool) $this->input('is_public', false),
            sort: (int) $this->input('sort'),
            contents: $this->getContents(),
            categoryId: $categoryId,
        );
    }

    private function getContents(): Contents
    {
        $contents = new Contents();
        foreach ($this->input('content', []) as $languageId => $content) {
            $contents->addContent(new Content(
                languageId: (int) $languageId,
                title:      $content['title'],
                content:    $content['content'] ?? '',
                storages:   $this->contentStorages($content),
                isTranslateAutomatically: (bool) $this->input('translate-automatically.' . $languageId, false),
            ));
        }
        return $contents;
    }

    private function contentStorages(array $content): Storages
    {
        $storages = new Storages();

        $images = $content['content_images'] ?? [];
        $storages->addMany($images, StorageType::ContentImages);

        return $storages;
    }
}
