<?php declare(strict_types=1);

namespace App\Http\Requests\Admin\Projects;

use App\Contracts\FormRequestDto;
use App\Dto\Service\Admin\Project\Language;
use App\Dto\Service\Admin\Project\Languages;
use App\Dto\Service\Admin\Project\StoreUpdate;
use App\Dto\Service\Storage\Storages;
use App\Enums\Lang;
use App\Enums\StorageType;
use App\Rules\HttpHost;
use Illuminate\Foundation\Http\FormRequest;
use Illuminate\Validation\Rules\Enum;

class StoreUpdateRequest extends FormRequest implements FormRequestDto
{
    /**
     * Get the validation rules that apply to the request.
     */
    public function rules(): array
    {
        $projectId = $this->project ?? null;

        return [
            'name' => ['required', 'string', 'max:255'],
            'code' => ['required', 'string', 'min:3', 'max:255', 'regex:/^[a-z0-9_-]+$/'],
            'http_host' => ['nullable', 'string', 'max:255', new HttpHost(), 'unique:projects,http_host,' . $projectId],
            'is_public' => ['required', 'boolean'],

            'logo.file' => ['nullable', 'numeric', 'min:1'],
            'logo.delete' => ['nullable', 'boolean'],

            'languages.items.*.id' => ['nullable', 'numeric'],
            'languages.items.*.title' => ['required', 'string', 'max:255'],
            'languages.items.*.code' => ['required', 'string', 'min:2', 'max:30', 'regex:/^[a-z_]+$/'],
            'languages.items.*.sort' => ['required', 'numeric', 'min:-1000', 'max:1000'],
            'languages.items.*.system_lang' => ['nullable', new Enum(Lang::class)],
            'languages.items.*.iso_code' => ['nullable', 'string', 'max:30', 'regex:/^[a-zA-Z-]+$/'],
            'languages.default' => ['required', 'numeric', function (string $attribute, mixed $value, \Closure $fail) {
                $languages = $this->input('languages.items', []);
                if (!isset($languages[$value])) {
                    $this->validator->getMessageBag()->add('languages.default', __('validation.required', ['attribute' => __('validation.attributes.language-default')]));
                }
            }],
            'languages.items' => ['required', 'array', function (string $attribute, mixed $value, \Closure $fail) {
                $values = [];
                foreach ($value as $index => $item) {
                    if (!isset($item['code'])) {
                        continue;
                    }
                    if (in_array($item['code'], $values)) {
                        $this->validator->getMessageBag()->add('languages.items.' . $index . '.code', __('validation.unique', ['attribute' => __('validation.attributes.language-code')]));
                    }
                    $values[] = $item['code'];
                }
            }],
        ];
    }


    public function getDto(): StoreUpdate
    {
        return new StoreUpdate(
            name: $this->input('name'),
            code: $this->input('code'),
            isPublic: (bool) $this->input('is_public', false),
            languages: $this->languages(),
            storages: $this->storages(),
            httpHost: $this->input('http_host', null),
        );
    }

    private function languages(): Languages
    {
        $languages = new Languages();

        $default = $this->input('languages.default', null);
        if ($default !== null) {
            $default = (int) $default;
        }

        foreach ($this->input('languages.items', []) as $index => $lang) {
            $languageId = $lang['id'] ?? null;
            if ($languageId !== null) {
                $languageId = (int) $languageId;
            }
            $systemLang = null;
            if ($lang['system_lang'] !== null) {
                $systemLang = Lang::tryFrom((int) $lang['system_lang']);
            }
            $language = new Language(
                title:     $lang['title'],
                code:      $lang['code'],
                sort:      (int) $lang['sort'],
                isDefault: ($default === $index),
                isoCode: $lang['iso_code'] ?? null,
                systemLang: $systemLang,
                id: $languageId,
            );
            $languages->addLanguage($language);
        }

        return $languages;
    }

    private function storages(): Storages
    {
        $storages = new Storages();

        $logo = $this->get('logo', []);
        $storages->add($logo, StorageType::Logo);

        return $storages;
    }
}
