<?php declare(strict_types=1);

namespace App\Repositories;

use App\Contracts\Search;
use App\Models\DocumentationCategory;
use App\Models\DocumentationVersion;
use App\Models\ProjectLanguage;
use App\Services\DocumentationCategory\BuilderCommand;
use App\Services\Search\CreateSearchInstanceCommand;
use App\Dto\Builder\DocumentationCategory as DocumentationCategoryBuilderDto;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Support\Str;

final readonly class DocumentationCategoryRepository
{
    public function __construct(
        private CreateSearchInstanceCommand $createSearchInstanceCommand,
        private BuilderCommand $builderCommand
    ) { }

    public function getCategories(int $versionId, DocumentationCategoryBuilderDto $documentationCategoryBuilderDto, array $with = []): Search
    {
        $query = $this->builderCommand->execute(
            query: DocumentationCategory::query()->where('version_id', $versionId)->with($with),
            documentationCategoryBuilderDto: $documentationCategoryBuilderDto
        );

        return $this->createSearchInstanceCommand->execute($query);
    }

    public function getCategoryById(int $id): ?DocumentationCategory
    {
        return DocumentationCategory::query()->where('id', $id)->first();
    }

    public function getCategoryBySlugWithContent(string $slug, int $versionId, ProjectLanguage $language): ?DocumentationCategory
    {
        $with = [
            'content' => function (HasOne $hasOne) use ($language) {
                $hasOne->where('language_id', $language->id);
            }
        ];

        return DocumentationCategory::query()
            ->where('version_id', $versionId)
            ->where('slug', $slug)
            ->with($with)
            ->first();
    }

    public function isExistsSlug(int $versionId, string $slug, ?int $exceptId = null): bool
    {
        return DocumentationCategory::query()
            ->where('version_id', $versionId)
            ->where('slug', Str::lower($slug))
            ->when($exceptId, function (Builder $query, int $exceptId) {
                $query->where('id', '!=', $exceptId);
            })
            ->withTrashed()
            ->exists();
    }

    public function getForSelect(DocumentationVersion $version, ?ProjectLanguage $defaultLanguage, ?DocumentationCategory $exceptCategory = null, array $withExcepts = []): array
    {
        $with = [
            'content' => function (HasOne $hasOne) use ($defaultLanguage) {
                $hasOne->when($defaultLanguage, function (Builder $query, ProjectLanguage $defaultLanguage) {
                    $query->where('language_id', $defaultLanguage->id);
                });
            }
        ];

        $categories = $version->categories()
            ->with($with)
            ->when($exceptCategory, function (Builder $query, DocumentationCategory $exceptCategory) {
                $query->whereNotIn(
                    'id',
                    $exceptCategory->descendantsAndSelf()->pluck('id')->toArray()
                );
            })
            ->when($withExcepts, function (Builder $query) use ($withExcepts) {
                $query->withTrashed()->whereNull('deleted_at')->orWhereIn('id', $withExcepts);
            })->get();

        return $categories->map(function (DocumentationCategory $documentationCategory) use ($defaultLanguage) {
                return [
                    'id'    => $documentationCategory->id,
                    'title' => $documentationCategory->content?->title ?? $documentationCategory->slug,
                ];
            })
            ->pluck('title', 'id')
            ->toArray();
    }
}
