<?php declare(strict_types=1);

namespace App\Services\Admin\Project;

use App\Dto\QuerySettingsDto;
use App\Dto\Builder\DocumentationCategory as DocumentationCategoryBuilderDto;
use App\Dto\Service\Admin\Project\DocumentationCategory\StoreUpdate;
use App\Dto\Service\Admin\Project\DocumentationCategoryContent\Content;
use App\Dto\Service\Admin\Project\DocumentationCategoryContent\Contents;
use App\Exceptions\Services\DocumentationCategory\ParentException;
use App\Exceptions\Services\ServiceException;
use App\Jobs\Translate\ProcessProjectDocumentationCategoryContent;
use App\Models\DocumentationCategory;
use App\Models\DocumentationCategoryContent;
use App\Models\ProjectLanguage;
use App\Models\User;
use App\Repositories\DocumentationCategoryRepository;
use App\Repositories\DocumentationVersionRepository;
use App\ServiceResults\ServiceResultArray;
use App\ServiceResults\ServiceResultError;
use App\ServiceResults\ServiceResultSuccess;
use App\ServiceResults\StoreUpdateResult;
use App\Services\DocumentationCategory\DocumentationCategoryCommandHandler;
use App\Services\DocumentationCategoryContent\ModelSyncCommand;
use App\Services\Service;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Support\Facades\DB;

final class DocumentationCategoryService extends Service
{
    public function __construct(
        private readonly DocumentationVersionRepository      $documentationVersionRepository,
        private readonly DocumentationCategoryRepository     $documentationCategoryRepository,
        private readonly DocumentationCategoryCommandHandler $categoryCommandHandler,
        private readonly ModelSyncCommand                    $contentSaveCommand,
    ) { }

    public function index(int $projectId, int $versionId, DocumentationCategoryBuilderDto $documentationCategoryBuilderDto, QuerySettingsDto $querySettingsDto, User $user): ServiceResultError | ServiceResultArray
    {
        $version = $this->documentationVersionRepository->getVersionById($versionId);
        $project = $version?->project;
        if (\is_null($version) || $project?->id !== $projectId) {
            return $this->errNotFound(__('Not Found'));
        }

        if ($user->cannot('viewAny', DocumentationCategory::class)) {
            return $this->errFobidden(__('Access is denied'));
        }

        $defaultLanguage = $project->languages()->where('is_default', 1)->first();
        $with = [
            'content' => function (HasOne $hasOne) use ($defaultLanguage) {
                /** @var ?ProjectLanguage $defaultLanguage */
                $hasOne->when($defaultLanguage, function (Builder $query, ProjectLanguage $defaultLanguage) {
                    $query->where('language_id', $defaultLanguage->id);
                });
            }
        ];
        $with = array_merge($with, $querySettingsDto->getQueryWith());

        $categories = $this->documentationCategoryRepository->getCategories(
            $version->id,
            $documentationCategoryBuilderDto,
            $with
        )->pagination(
            $querySettingsDto->getLimit(),
            $querySettingsDto->getPage()
        );

        return $this->result([
            'version' => $version,
            'project' => $project,
            'categories' => $categories,
        ]);
    }

    public function create(int $projectId, int $versionId, User $user): ServiceResultError | ServiceResultArray
    {
        $version = $this->documentationVersionRepository->getVersionById($versionId);
        $project = $version?->project;
        if (\is_null($version) || $project?->id !== $projectId) {
            return $this->errNotFound(__('Not Found'));
        }

        if ($user->cannot('create', DocumentationCategory::class)) {
            return $this->errFobidden(__('Access is denied'));
        }

        $defaultLanguage = $project->languages->where('is_default', 1)->first();
        return $this->result([
            'version'  => $version,
            'project'  => $project,
            'category' => new DocumentationCategory(),
            'categories' => $this->documentationCategoryRepository->getForSelect($version, $defaultLanguage),
            'serviceTranslationEnable' => config('translation_service.enable', false),
        ]);
    }

    public function edit(int $projectId, int $versionId, int $categoryId, User $user): ServiceResultError | ServiceResultArray
    {
        $version = $this->documentationVersionRepository->getVersionById($versionId);
        $project = $version?->project;
        if (\is_null($version) || $project?->id !== $projectId) {
            return $this->errNotFound(__('Not Found'));
        }

        $category = $this->documentationCategoryRepository->getCategoryById($categoryId);
        if (\is_null($category)) {
            return $this->errNotFound(__('Not Found'));
        }

        if ($user->cannot('view', $category)) {
            return $this->errFobidden(__('Access is denied'));
        }

        $withCategories = [];
        if ($category->parent_id) {
            $withCategories[] = $category->parent_id;
        }
        $defaultLanguage = $project->languages->where('is_default', 1)->first();
        return $this->result([
            'version'  => $version,
            'project'  => $project,
            'category' => $category,
            'categories' => $this->documentationCategoryRepository->getForSelect($version, $defaultLanguage, $category, $withCategories),
            'serviceTranslationEnable' => config('translation_service.enable', false),
        ]);
    }

    public function store(int $projectId, int $versionId, StoreUpdate $data, User $user): ServiceResultError | StoreUpdateResult
    {
        if ($user->cannot('create', DocumentationCategory::class)) {
            return $this->errFobidden(__('Access is denied'));
        }

        $version = $this->documentationVersionRepository->getVersionById($versionId);
        $project = $version?->project;
        if (\is_null($version) || $project?->id !== $projectId) {
            return $this->errNotFound(__('Not Found'));
        }

        try {
            $category = DB::transaction(function () use ($data, $version, $user, $versionId, $project) {
                if ($this->documentationCategoryRepository->isExistsSlug($versionId, $data->getSlug()) !== false) {
                    $error = $this->errValidate(
                        __('validation.unique', ['attribute' => __('validation.attributes.slug')]),
                        ['slug' => __('validation.unique', ['attribute' => __('validation.attributes.slug')])]
                    );
                    throw new ServiceException($error);
                }

                $dataCategory = $this->getDataCategory($data);
                $category = $this->categoryCommandHandler->handleStore($version, $dataCategory);
                $this->contentSaveCommand->execute($project, $category, $data->getContents());

                return $category;
            });

            if (\config('translation_service.enable', false)) {
                $this->translateContent($category, $data->getContents());
            }
        } catch (ServiceException $e) {
            return $e->getServiceResultError();
        } catch (\Throwable $e) {
            report($e);
            return $this->errService(__('Server Error'));
        }

        return $this->resultStoreUpdateModel($category, __('Category successfully created'));
    }

    public function update(int $projectId, int $versionId, int $categoryId, StoreUpdate $data, User $user): ServiceResultError | StoreUpdateResult
    {
        $version = $this->documentationVersionRepository->getVersionById($versionId);
        $project = $version?->project;
        if (\is_null($version) || $project?->id !== $projectId) {
            return $this->errNotFound(__('Not Found'));
        }

        $category = $this->documentationCategoryRepository->getCategoryById($categoryId);
        if (\is_null($category)) {
            return $this->errNotFound(__('Not Found'));
        }

        if ($user->cannot('view', $category)) {
            return $this->errFobidden(__('Access is denied'));
        }

        if ($data->getParentId() === $category->id) {
            return $this->errValidate(
                __('validation.parent', ['attribute' => __('validation.attributes.parent_id')]),
                ['parent_id' => __('validation.parent', ['attribute' => __('validation.attributes.parent_id')])]
            );
        }

        try {
            $category = DB::transaction(function () use ($data, $category, $versionId, $project) {
                if ($this->documentationCategoryRepository->isExistsSlug($versionId, $data->getSlug(), $category->id) !== false) {
                    $error = $this->errValidate(
                        __('validation.unique', ['attribute' => __('validation.attributes.slug')]),
                        ['slug' => __('validation.unique', ['attribute' => __('validation.attributes.slug')])]
                    );
                    throw new ServiceException($error);
                }

                $dataCategory = $this->getDataCategory($data);
                $category = $this->categoryCommandHandler->handleUpdate($category, $dataCategory);
                $this->contentSaveCommand->execute($project, $category, $data->getContents());

                return $category;
            });

            if (\config('translation_service.enable', false)) {
                $this->translateContent($category, $data->getContents());
            }
        } catch (ServiceException $e) {
            return $e->getServiceResultError();
        } catch (ParentException $e) {
            return $this->errValidate(
                __('validation.parent_cycle_detected', ['attribute' => __('validation.attributes.parent_id')]),
                ['parent_id' => __('validation.parent_cycle_detected', ['attribute' => __('validation.attributes.parent_id')])]
            );
        } catch (\Throwable $e) {
            report($e);
            return $this->errService(__('Server Error'));
        }

        return $this->resultStoreUpdateModel($category, __('Category updated successfully'));
    }

    public function destroy(int $projectId, int $versionId, int $categoryId, User $user): ServiceResultError|ServiceResultSuccess
    {
        $version = $this->documentationVersionRepository->getVersionById($versionId);
        $project = $version?->project;
        if (\is_null($version) || $project?->id !== $projectId) {
            return $this->errNotFound(__('Not Found'));
        }

        $category = $this->documentationCategoryRepository->getCategoryById($categoryId);
        if (\is_null($category)) {
            return $this->errNotFound(__('Not Found'));
        }

        if ($user->cannot('delete', $category)) {
            return $this->errFobidden(__('Access is denied'));
        }

        try {
            DB::transaction(function () use ($category) {
                $this->categoryCommandHandler->handleDestroy($category);
            });
        } catch (\Throwable $e) {
            report($e);
            return $this->errService(__('Server Error'));
        }

        return $this->ok(__('Category successfully deleted'));
    }

    private function getDataCategory(StoreUpdate $data): array
    {
        return [
            'slug'      => $data->getSlug(),
            'is_public' => $data->isPublic(),
            'sort'      => $data->getSort(),
            'parent_id' => $data->getParentId(),
        ];
    }

    private function translateContent(DocumentationCategory $category, Contents $contents): void
    {
        $translateExceptLanguages = [];

        $translateLanguages = [];
        foreach ($contents->getContents() as $content) {
            /** @var Content $content */
            $translateExceptLanguages[] = $content->getLanguageId();
            if ($content->isTranslateAutomatically()) {
                $translateLanguages[] = $content->getLanguageId();
            }
        }

        if (empty($translateLanguages)) {
            return;
        }

        $translateContentIds = $category->contents()->select('id')
            ->whereIn('language_id', $translateLanguages)
            ->get()->pluck('id')->toArray();
        ProcessProjectDocumentationCategoryContent::dispatch($category->id, $translateContentIds, $translateExceptLanguages);
    }
}
