<?php declare(strict_types=1);

namespace App\Services\Admin\Project;

use App\Dto\Service\Admin\Project\ServiceTranslate\Update;
use App\Models\User;
use App\Repositories\ProjectRepository;
use App\ServiceResults\ServiceResultArray;
use App\ServiceResults\ServiceResultError;
use App\ServiceResults\ServiceResultSuccess;
use App\Services\ProjectTranslationService\ModelSyncCommand as TranslationServiceModelSyncCommand;
use App\Services\Service;
use Illuminate\Support\Facades\DB;

final class ServiceTranslateService extends Service
{
    public function __construct(
        private readonly ProjectRepository                  $projectRepository,
        private readonly TranslationServiceModelSyncCommand $translationServiceModelSyncCommand,
    ) { }

    public function view(int $projectId, User $user): ServiceResultError | ServiceResultArray
    {
        $project = $this->projectRepository->getProjectById($projectId);
        if (\is_null($project)) {
            return $this->errNotFound(__('Not Found'));
        }

        if (
            config('translation_service.enable', false) === false
            || $user->cannot('settingUpAutomaticTranslation', $project)
        ) {
            return $this->errFobidden(__('Access is denied'));
        }

        return $this->result([
            'project'   => $project,
            'languages' => $project->languages()->with(['serviceTranslate'])->get(),
        ]);
    }

    public function update(int $projectId, Update $data, User $user): ServiceResultError | ServiceResultSuccess
    {
        $project = $this->projectRepository->getProjectById($projectId);
        if (\is_null($project)) {
            return $this->errNotFound(__('Not Found'));
        }

        if (
            config('translation_service.enable', false) === false
            || $user->cannot('settingUpAutomaticTranslation', $project)
        ) {
            return $this->errFobidden(__('Access is denied'));
        }

        try {
            DB::transaction(function () use ($data, $project) {
                $this->translationServiceModelSyncCommand->execute($project, $data->getTranslations());
            });
        } catch (\Throwable $e) {
            report($e);
            return $this->errService(__('Server Error'));
        }

        return $this->ok(__('The settings were saved successfully'));
    }
}
