<?php declare(strict_types=1);

namespace App\Services\Translate\Completed\Project;

use App\Dto\Service\ProjectTranslationServiceHash\Hashes;
use App\Exceptions\Services\Translate\CompletedException;
use App\Jobs\Translate\ProcessProjectDocumentationCategoryContent;
use App\Models\DocumentationCategory;
use App\Models\DocumentationCategoryContent;
use App\Repositories\DocumentationCategoryRepository;
use App\Services\ProjectTranslationService\NoTranslateAttributeHandler;
use App\Services\ProjectTranslationServiceHash\CompletionChecker;
use Illuminate\Support\Facades\DB;
use KorElf\TranslateLaravel\Contracts\TranslationCompletedListener;

final readonly class DocumentationCategoryContentService implements TranslationCompletedListener
{
    public function __construct(
        private DocumentationCategoryRepository $categoryRepository,
        private NoTranslateAttributeHandler     $noTranslateAttributeHandler,
        private CompletionChecker               $completionChecker,
    ) { }

    /**
     * @throws CompletedException
     */
    public function onTranslationCompleted(array $translatedText, array $data = []): void
    {
        if (
            !isset($data['categoryId'])
            || !isset($data['languageId'])
            || !isset($data['hashes'])
        ) {
            throw new CompletedException('Required data is missing: categoryId, languageId, or hashes.');
        }

        $category = $this->categoryRepository->getCategoryById((int) $data['categoryId']);
        if ($category === null) {
            return;
        }

        $categoryContent = DB::transaction(function () use ($data, $category, $translatedText) {
            $values = [];
            $hashes = $this->completionChecker->execute(
                $this->getHashes($data['hashes']),
            );

            foreach ($translatedText as $translatedTextKey => $translatedTextValue) {
                if ($hashes->isStatusWaiting($translatedTextKey) !== true) {
                    continue;
                }

                $translatedTextValue = $this->noTranslateAttributeHandler->handleRemoveAttribute($translatedTextValue);
                $values[$translatedTextKey] = $translatedTextValue;
            }
            if (\count($values) === 0) {
                return null;
            }

            $categoryContent = $category->contents()->where('language_id', $data['languageId'])->first();
            if ($categoryContent !== null) {
                $categoryContent->update($values);
                return $categoryContent;
            }

            $values['language_id'] = $data['languageId'];
            return $category->contents()->create($values);
        });
        if (\config('translation_service.enable', false) && $categoryContent !== null) {
            $this->translateContent($category, $categoryContent, $data);
        }
    }

    private function translateContent(DocumentationCategory $category, DocumentationCategoryContent $categoryContent, array $data): void
    {
        $translateExceptLanguages = $data['exceptLanguages'] ?? [];
        $translateExceptLanguages[] = $data['languageId'];
        ProcessProjectDocumentationCategoryContent::dispatch($category->id, [$categoryContent->id], $translateExceptLanguages);
    }

    private function getHashes(array $hashes): Hashes
    {
        $hashesDto = new Hashes();
        foreach ($hashes as $field => $hash) {
            $hashesDto->add((int) $hash['hashId'], $field, $hash['hash']);
        }
        return $hashesDto;
    }
}
