<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Support\Facades\Schema;

return new class extends Migration
{
    /**
     * Run the migrations.
     */
    public function up(): void
    {
        Schema::create('projects', function (Blueprint $table) {
            $table->id();
            $table->string('name')->index();
            $table->string('code')->unique();
            $table->string('http_host')->nullable()->unique()->comment('Scheme and Domain and Port (example: http://localhost:8080).');
            $table->boolean('is_public')->default(0)->index();
            $table->timestamps();
            $table->softDeletes()->index();
            $table->index('created_at');
            $table->index(['is_public', 'deleted_at']);
        });

        Schema::table('roles', function (Blueprint $table) {
            $table->unsignedInteger('morph_type')->nullable();
            $table->unsignedBigInteger('morph_id')->nullable();
            $table->index(['morph_type', 'morph_id']);
            $table->dropUnique(['code']);
            $table->unique(['code', 'morph_type', 'morph_id']);
        });

        Schema::create('storage', function (Blueprint $table) {
            $table->id();
            $table->string('file');
            $table->unsignedInteger('morph_type');
            $table->unsignedBigInteger('morph_id')->nullable()->index();
            $table->unsignedInteger('type')->comment('File type (main photo, video, gallery, etc.).');
            $table->json('data')->nullable();
            $table->timestamps();
            $table->softDeletes();
            $table->unsignedBigInteger('created_user_id')->nullable();
            $table->foreign('created_user_id')->references('id')->on('users');
            $table->index(['morph_type', 'morph_id']);
        });

        Schema::create('project_languages', function (Blueprint $table) {
            $table->id();
            $table->unsignedBigInteger('project_id')->index();
            $table->foreign('project_id')->references('id')->on('projects');
            $table->string('code');
            $table->string('title');
            $table->boolean('is_default');
            $table->integer('sort')->index();
            $table->timestamps();
            $table->softDeletes();
            $table->index(['project_id', 'deleted_at']);
            $table->unique(['project_id', 'code']);
        });

        Schema::create('project_content', function (Blueprint $table) {
            $table->id();
            $table->string('title')->index();
            $table->unsignedBigInteger('project_id')->index();
            $table->foreign('project_id')->references('id')->on('projects');
            $table->unsignedBigInteger('language_id')->index();
            $table->foreign('language_id')->references('id')->on('project_languages');
            $table->longText('description');
            $table->timestamps();
            $table->softDeletes();
            $table->index(['project_id', 'language_id', 'deleted_at']);
        });

        Schema::create('project_links', function (Blueprint $table) {
            $table->id();
            $table->string('title');
            $table->string('link');
            $table->unsignedBigInteger('project_id')->index();
            $table->foreign('project_id')->references('id')->on('projects');
            $table->unsignedBigInteger('language_id')->nullable()->index();
            $table->foreign('language_id')->references('id')->on('project_languages');
            $table->index(['project_id', 'language_id', 'deleted_at']);
            $table->integer('sort')->index();
            $table->softDeletes();
        });
    }

    /**
     * Reverse the migrations.
     */
    public function down(): void
    {
        Schema::dropIfExists('project_links');
        Schema::dropIfExists('project_content');
        Schema::dropIfExists('project_languages');
        Schema::dropIfExists('storage');
        Schema::table('roles', function (Blueprint $table) {
            $table->dropUnique(['code', 'morph_type', 'morph_id']);
            $table->dropColumn('morph_type');
            $table->dropColumn('morph_id');
            $table->unique('code');
        });
        Schema::dropIfExists('projects');
    }
};
